@@ -201,8 +201,13 @@ class Balloon:
             return (0, 0)
 
     def set_balloon_default(self):
+        default_id = self.request_parent('GET', 'get_balloon_default_id')
+        try:
+            default_id = int(default_id)
+        except:
+            default_id = 0
         for side in range(len(self.window)):
-            self.window[side].set_balloon(0) ## FIXME
+            self.window[side].set_balloon(default_id)
 
     def set_balloon(self, side, num):
         if len(self.window) > side:
@@ -222,6 +227,10 @@ class Balloon:
         else:
             return (0, 0)
 
+    def set_autoscroll(self, flag):
+        for side in range(len(self.window)):
+            self.window[side].set_autoscroll(flag)
+
     def is_shown(self, side):
         return 0 if len(self.window) <= side else self.window[side].is_shown()
 
@@ -361,6 +370,12 @@ class Balloon:
             self.passwordinputbox.set_limittime(limittime)
             self.passwordinputbox.show(default)
 
+    def close_inputbox(self, symbol):
+        if not self.user_interaction:
+            return
+        self.inputbox.close(symbol)
+        self.passwordinputbox.close(symbol)
+
 
 class BalloonWindow:
 
@@ -383,6 +398,7 @@ class BalloonWindow:
         self.__font_name = ''
         self.text_count = 0
         self.balloon_surface = None
+        self.autoscroll = True
         # create drawing area
         self.darea = Gtk.DrawingArea()
         self.darea.set_events(Gdk.EventMask.EXPOSURE_MASK|
@@ -412,6 +428,10 @@ class BalloonWindow:
             text_g = 255 - text_g
             text_b = 255 - text_b
         self.text_active_color = (text_r / 255., text_g / 255., text_b / 255.)
+        sstp_r = desc.get_with_type('sstpmessage.font.color.r', int, text_r)
+        sstp_g = desc.get_with_type('sstpmessage.font.color.g', int, text_g)
+        sstp_b = desc.get_with_type('sstpmessage.font.color.b', int, text_b)
+        self.sstp_message_color = (sstp_r / 255., sstp_g / 255., sstp_b / 255.)
         # initialize
         self.direction = min(side, 1) ## kluge: multi character
         self.position = (0, 0)
@@ -610,8 +630,8 @@ class BalloonWindow:
         self.reset_arrow()
         self.reset_sstp_marker()
         self.reset_message_regions()
+        self.request_parent('NOTIFY', 'position_balloons')
         if self.__shown:
-            self.request_parent('NOTIFY', 'position_balloons')
             self.darea.queue_draw()
 
     def set_direction(self, dir):
@@ -619,6 +639,9 @@ class BalloonWindow:
             self.direction = dir # 0: left, 1: right
             self.reset_balloon()
 
+    def set_autoscroll(self, flag):
+        self.autoscroll = bool(flag)
+
     def config_adjust(self, name, base, default_value):
         path, config = self.balloon[self.balloon_id]
         value = config.get_with_type(name, int)
@@ -746,7 +769,7 @@ class BalloonWindow:
         # draw sstp message
         x, y, w, h = self.sstp_region
         self.sstp_layout.set_text(self.sstp_message, -1)
-        cr.set_source_rgba(*self.text_normal_color)
+        cr.set_source_rgba(*self.sstp_message_color)
         cr.move_to(x, y)
         PangoCairo.update_layout(cr, self.sstp_layout)
         PangoCairo.show_layout(cr, self.sstp_layout)
@@ -1233,9 +1256,10 @@ class BalloonWindow:
                         del cr
         else:
             self.darea.queue_draw()
-            while line >= self.lineno + self.lines:
-                self.lineno += 1
-                self.darea.queue_draw()
+            if self.autoscroll:
+                while line >= self.lineno + self.lines:
+                    self.lineno += 1
+                    self.darea.queue_draw()
 
 
 class CommunicateWindow(metaclass=abc.ABCMeta):
@@ -1488,6 +1512,14 @@ class InputBox(CommunicateWindow):
     def cancel(self):
         self.send(None, cancel=True)
 
+    def close(self, symbol):
+        if self.symbol is None:
+            return
+        if symbol != '__SYSTEM_ALL_INPUT__' and self.symbol != symbol:
+            return
+        self.window.hide()
+        self.cancel()
+
     def send(self, data, cancel=False, timeout=False):
         if self.timeout_id is not None:
             GLib.source_remove(self.timeout_id)            
@@ -1497,7 +1529,7 @@ class InputBox(CommunicateWindow):
         if cancel:
             self.request_parent('NOTIFY', 'notify_event',
                                 'OnUserInputCancel', '', 'cancel')
-        elif timout and \
+        elif timeout and \
                 self.request_parent('GET', 'notify_event',
                                     'OnUserInputCancel', '', 'timeout'):
             pass
@@ -1509,6 +1541,7 @@ class InputBox(CommunicateWindow):
             pass
         elif self.request_parent('GET', 'notify_event', self.symbol, data):
             pass
+        self.symbol = None
         self.request_parent('NOTIFY', 'reset_user_interaction')
 
 
