@@ -236,9 +236,22 @@ class Menu:
 
     def create_mayuna_menu(self, mayuna_menu):
         self.__mayuna_menu = []
-        for side, index in [('sakura', 0), ('kero', 1)]:
+        for side in mayuna_menu:
+            if side == 'sakura':
+                index = 0
+            elif side == 'kero':
+                index = 1
+            elif side.startswith('char'):
+                try:
+                    index = int(side[4:])
+                except:
+                    continue
+            else:
+                continue
+            for _ in range(len(self.__mayuna_menu), index + 1):
+                self.__mayuna_menu.append(None)
             if mayuna_menu[side]:
-                self.__mayuna_menu.append(Gtk.Menu())
+                self.__mayuna_menu[index] = Gtk.Menu()
                 item = Gtk.TearoffMenuItem()
                 item.show()
                 self.__mayuna_menu[index].append(item)
@@ -257,8 +270,6 @@ class Menu:
                         item = Gtk.SeparatorMenuItem()
                     item.show()
                     self.__mayuna_menu[index].append(item)
-            else:
-                self.__mayuna_menu.append(None)
 
     __re_shortcut = re.compile(r'&(?=[\x21-\x7e])')
 
