@@ -81,10 +81,11 @@ class Sakura:
     PAUSE_TIMEOUT  = 30  # [sec]
     SILENT_TIME    = 15  # [sec]
     # script modes
-    BROWSE_MODE = 1
-    SELECT_MODE = 2
-    PAUSE_MODE  = 3
-    WAIT_MODE   = 4 
+    BROWSE_MODE        = 1
+    SELECT_MODE        = 2
+    PAUSE_MODE         = 3
+    WAIT_MODE          = 4 
+    PAUSE_NOCLEAR_MODE = 5
     # script origins
     FROM_SSTP_CLIENT = 1
     FROM_GHOST       = 2
@@ -193,6 +194,12 @@ class Sakura:
         assert key in self.shells
         del self.shells[key]
 
+    def notify_installedshellname(self):
+        installed = []
+        for key in self.shells:
+            installed.append(self.shells[key].baseinfo[0])
+        self.notify_event('installedshellname', *installed)
+
     def get_shell_menu(self):
         current_key = self.get_current_shell()
         for key in self.shells:
@@ -328,6 +335,7 @@ class Sakura:
             logging.error('{0} cannot load SHIORI({1})'.format(
                     self.get_selfname(), self.shiori_name))
         self.__charset = 'Shift_JIS' # default
+        self.get_event_response('OnInitialize', event_type='NOTIFY')
         self.get_event_response('basewareversion',
                                 ninix.version.VERSION,
                                 'ninix-aya',
@@ -340,6 +348,7 @@ class Sakura:
                 proc(flag_break=False)
             self.script_finally = []
         if not self.__temp_mode:
+            self.get_event_response('OnDestroy', event_type='NOTIFY')
             self.shiori.unload()
         self.stop()
 
@@ -456,7 +465,7 @@ class Sakura:
         return self.__running
 
     def is_paused(self):
-        return self.script_mode == self.PAUSE_MODE
+        return self.script_mode in {self.PAUSE_MODE, self.PAUSE_NOCLEAR_MODE}
 
     def is_talking(self):
         return int(bool(self.processed_script or self.processed_text))
@@ -834,6 +843,8 @@ class Sakura:
             self.balloon.clear_text_all()
             self.balloon.hide_all()
             self.script_side = 0
+        elif self.script_mode == self.PAUSE_NOCLEAR_MODE:
+            self.script_mode = self.BROWSE_MODE
         elif button == 1 and click == 1:
             self.raise_all()
         if self.vanished:
@@ -1055,6 +1066,9 @@ class Sakura:
             default = list(self.shells.keys())[0] # XXX
         return default
 
+    def get_balloon_default_id(self):
+        return self.desc.get('balloon.defaultsurface', '0')
+
     def select_shell(self, shell_key):
         assert self.shells and shell_key in self.shells
         self.shell_directory = shell_key # save user's choice
@@ -1316,7 +1330,7 @@ class Sakura:
             self.old_otherghostname = otherghostname
         if not self.__running:
             pass
-        elif self.script_mode == self.PAUSE_MODE:
+        elif self.script_mode in {self.PAUSE_MODE, self.PAUSE_NOCLEAR_MODE}:
             ##if idle > self.PAUSE_TIMEOUT:
             ##    self.script_mode = self.BROWSE_MODE
             pass
@@ -1690,7 +1704,10 @@ class Sakura:
 
     def __yen_x(self, args):
         if self.script_mode == self.BROWSE_MODE:
-            self.script_mode = self.PAUSE_MODE
+            if len(args) > 0 and self.expand_meta(args[0]) == 'noclear':
+                self.script_mode = self.PAUSE_NOCLEAR_MODE
+            else:
+                self.script_mode = self.PAUSE_MODE
 
     def __yen_a(self, args):
         self.start_script(self.getaistringrandom())
@@ -1823,6 +1840,8 @@ class Sakura:
                     self.balloon.open_passwordinputbox(args[2])
         elif args[0:2] == ['open', 'configurationdialog']:
             self.request_parent('NOTIFY', 'edit_preferences')
+        elif args[0:2] == ['close', 'inputbox'] and argc > 2:
+            self.balloon.close_inputbox(args[2])
         elif args[0:2] == ['change', 'balloon'] and argc > 2:
             key = self.request_parent('GET', 'find_balloon_by_name', args[2])
             if key is not None:
@@ -1897,6 +1916,13 @@ class Sakura:
                     self.surface.set_alignment(self.script_side, 2)
             elif args[2] == 'default':
                 self.surface.reset_alignment()
+        elif args[0:2] == ['set', 'autoscroll'] and argc > 2:
+            if args[2] == 'disable':
+                self.balloon.set_autoscroll(False)
+            elif args[2] == 'enable':
+                self.balloon.set_autoscroll(True)
+            else:
+                pass ## FIXME
         elif args[0:2] == ['set', 'windowstate'] and argc > 2:
             if args[2] == 'minimize':
                 self.surface.window_iconify(True)
@@ -2150,6 +2176,7 @@ class Sakura:
         self.time_critical_session = 0
         self.quick_session = 0
         self.set_synchronized_session(reset=1)
+        self.balloon.set_autoscroll(True)
         self.reset_idle_time()
 
     def set_synchronized_session(self, list=[], reset=0):
