@@ -196,7 +196,7 @@ class Parser:
                 self.script.append((SCRIPT_TEXT, tuple(text), self.column))
                 text = []
             if lexeme in ['\\a', '\\c', '\\e', '\\t', '\\_e',
-                          '\\v', '\\x', '\\y', '\\z', '\\_q',
+                          '\\v', '\\y', '\\z', '\\_q',
                           '\\4', '\\5', '\\6', '\\7',
                           '\\2', '\\*', '\\-', '\\+', '\\_+',
                           '\\_n', '\\_V', '\\__c', '\\__t',
@@ -229,7 +229,7 @@ class Parser:
                             '\\3', '\\8', '\\9']:
                 argument = self.read_sbra_text()
                 self.script.append((SCRIPT_TAG, lexeme, argument, self.column))
-            elif lexeme in ['\\n']:
+            elif lexeme in ['\\n', '\\x']:
                 if self.tokens and self.tokens[0][0] == TOKEN_OPENED_SBRA:
                     argument = self.read_sbra_text()
                     self.script.append((SCRIPT_TAG, lexeme, argument,
