@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -178,7 +178,8 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
                 entry_db, self.server)
             self.server.request_handler = self # keep alive
 
-    PROHIBITED_TAGS = [r'\j', r'\-', r'\+', r'\_+', r'\!', r'\8', r'\_v']
+    PROHIBITED_TAGS = [r'\j', r'\-', r'\+', r'\_+', r'\!', r'\8', r'\_v',
+                       r'\C']
 
     def check_script(self, script):
         if not self.local_request():
