@@ -176,11 +176,6 @@ class Surface:
         self.__tooltips = tooltips
         self.name = name
         self.prefix = prefix
-        if alias is None:
-            alias0 = alias1 = None
-        else:
-            alias0 = alias.get('sakura.surface.alias')
-            alias1 = alias.get('kero.surface.alias')
         # load surface
         surfaces = {}
         elements = {}
@@ -266,40 +261,14 @@ class Surface:
             key = match.group(1)
             # define animation patterns
             mayuna[key] = ninix.seriko.get_mayuna(config)
-        bind = {}
-        for side in ['sakura', 'kero']:
-            bind[side] = {}
-            for index in range(128):
-                name = self.desc.get(
-                    '{0}.bindgroup{1:d}.name'.format(side, index), None)
-                default = self.desc.get(
-                    '{0}.bindgroup{1:d}.default'.format(side, index), 0)
-                if name is not None:
-                    bind[side][index] = [name, default]
         self.mayuna = {}
-        for side in ['sakura', 'kero']:
-            self.mayuna[side] = []
-            for index in range(128):
-                key = self.desc.get('{0}.menuitem{1:d}'.format(side, index), None)
-                if key == '-':
-                    self.mayuna[side].append([key, None, 0])
-                else:
-                    try:
-                        key = int(key)
-                    except:
-                        pass
-                    else:
-                        if key in bind[side]:
-                            name = bind[side][key][0].split(',')
-                            self.mayuna[side].append([key, name[1],
-                                                      bind[side][key][1]])
         # create surface windows
         for surface_window in self.window:
             surface_window.destroy()
         self.window = []
         self.__surface = surface
-        self.add_window(0, default_sakura, alias0, mayuna, bind['sakura'])
-        self.add_window(1, default_kero, alias1, mayuna, bind['kero'])
+        self.add_window(0, default_sakura, alias, mayuna)
+        self.add_window(1, default_kero, alias, mayuna)
 
     def get_menu_pixmap(self):
         top_dir = self.prefix
@@ -350,7 +319,7 @@ class Surface:
         foreground = (fontcolor_r, fontcolor_g, fontcolor_b)
         return background, foreground
 
-    def add_window(self, side, default, alias=None, mayuna={}, bind={}):
+    def add_window(self, side, default_id, config_alias, mayuna={}):
         assert len(self.window) == side
         if side == 0:
             name = 'sakura'
@@ -363,6 +332,34 @@ class Surface:
         else:
             name = 'char{0:d}'.format(side)
             title = ''.join(('surface.', name))
+        if config_alias is None:
+            alias = None
+        else:
+            alias = config_alias.get('{}.surface.alias'.format(name))
+        # MAYUNA
+        bind = {}
+        for index in range(128):
+            group = self.desc.get(
+                '{0}.bindgroup{1:d}.name'.format(name, index), None)
+            default = self.desc.get(
+                '{0}.bindgroup{1:d}.default'.format(name, index), 0)
+            if group is not None:
+                bind[index] = [group, default]
+        self.mayuna[name] = []
+        for index in range(128):
+            key = self.desc.get('{0}.menuitem{1:d}'.format(name, index), None)
+            if key == '-':
+                self.mayuna[name].append([key, None, 0])
+            else:
+                try:
+                    key = int(key)
+                except:
+                    pass
+                else:
+                    if key in bind:
+                        group = bind[key][0].split(',')
+                        self.mayuna[name].append([key, group[1],
+                                                  bind[key][1]])
         skip_taskbar = bool(side >= 1)
         gtk_window = self.create_gtk_window(title, skip_taskbar)
         seriko = self.get_seriko(self.__surface)
@@ -371,7 +368,7 @@ class Surface:
             tooltips = self.__tooltips[name]
         surface_window = SurfaceWindow(
             gtk_window, side, self.desc, alias, self.__surface, tooltips,
-            self.__surfaces, seriko, self.__region, mayuna, bind, default)
+            self.__surfaces, seriko, self.__region, mayuna, bind, default_id)
         surface_window.set_responsible(self.handle_request)
         self.window.append(surface_window)
 
