@@ -644,6 +644,36 @@ class Application:
             elif filetype == 'kinoko':
                 self.kinoko = ninix.home.search_kinoko()
 
+    def notify_installedghostname(self, key=None):
+        installed = []
+        for value in self.ghosts.values():
+            sakura = value.instance
+            if sakura is None:
+                continue
+            installed.append(sakura.get_name(default=''))
+        if key is not None:
+            if key in self.ghosts:
+                sakura = self.ghosts[key].instance
+                sakura.notify_event('installedghostname', *installed)
+        else:
+            for sakura in self.get_working_ghost():
+                sakura.notify_event('installedghostname', *installed)
+
+    def notify_installedballoonname(self, key=None):
+        installed = []
+        for value in self.balloons.values():
+            desc, balloon = value.baseinfo
+            subdir = balloon['balloon_dir'][0]
+            installed.append(desc.get('name', os.fsdecode(subdir)))
+        if key is not None:
+            if key in self.ghosts:
+                sakura = self.ghosts[key].instance
+                sakura.notify_event('installedballoonname', *installed)
+        else:
+            for sakura in self.get_working_ghost():
+                sakura.notify_event('installedballoonname', *installed)
+
+
     @property
     def current_sakura_instance(self):
         return self.ghosts[self.current_sakura].instance
@@ -1085,6 +1115,9 @@ class Application:
         sakura.notify_preference_changed()
         sakura.start(key, init, temp, vanished, ghost_changed,
                      self_name, name, shell, last_script, abend)
+        self.notify_installedghostname(key)
+        self.notify_installedballoonname(key)
+        sakura.notify_installedshellname()
         self.set_menu_sensitive(key, False)
 
     def update_working(self, ghost_name):
@@ -1150,6 +1183,7 @@ class Application:
                 if sakura.is_running(): # stop if working
                     self.stop_sakura(sakura)
                 del self.ghosts[ghost_dir]
+        self.notify_installedghostname()
 
     def add_balloon(self, balloon_dir):
         if balloon_dir in self.balloons:
@@ -1171,6 +1205,7 @@ class Application:
             if exists:
                 del self.balloons[balloon_dir]
         self.balloon_menu = self.create_balloon_menu()
+        self.notify_installedballoonname()
 
     def vanish_sakura(self, sakura, next_ghost):
         # remove ghost
